// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

#include "saa.h"

#include "constants.h"
#include "geometry.h"

#include <cmath>
#include <hdf5.h>

namespace spex {

SAA::SAA(const std::string& filename)
{
    const hid_t h5 { H5Fopen(filename.c_str(), H5F_ACC_RDONLY, H5P_DEFAULT) };

    const hid_t h5_lat { H5Dopen(h5, "latitude", H5P_DEFAULT) };
    const hid_t dspace { H5Dget_space(h5_lat) };
    hsize_t dim;
    H5Sget_simple_extent_dims(dspace, &dim, NULL);
    lat.resize(dim);
    H5Dread(
      h5_lat, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, lat.data());
    H5Dclose(h5_lat);

    const hid_t h5_lon { H5Dopen(h5, "longitude", H5P_DEFAULT) };
    lon.resize(lat.size());
    H5Dread(
      h5_lon, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, lon.data());
    H5Dclose(h5_lon);

    H5Fclose(h5);

    for (int i {}; i < static_cast<int>(lat.size()); ++i) {
        lat[i] *= math::deg_to_rad;
        lon[i] *= math::deg_to_rad;
    }
}

[[nodiscard]] auto SAA::containsPoint(const double lat, const double lon) const
  -> bool
{
    return isInPolygonWN(lon, lat, this->lon, this->lat);
}

} // namespace spex
